#ifndef _FPGA_H
#define _FPGA_H

#include "Prefix.h"
#include "FPGA_Register.h"
#include "FPGA_Config.h"
#include "FPGA_Power.h"
#include "FPGA_Status.h"
#include "FPGA_DMA.h"
#include "FPGA_I2C.h"
#include "FPGA_ROM_.h"
#include "PCI_Device.h"
#include <stdio.h>
#include <stdlib.h>

namespace EARTH {

namespace PCI { class Device; }

namespace FPGA {
	class Register;
	class Config;
	class Power;
	class Status;
	class DMA;
	class I2C;
	class ROM_;

	class _ {
	public:
		struct Argument {
			PCI::Device		*Pci;
		};
	
		_(const Argument *argument) : mPCI(argument->Pci)
		{
			mRegister = NULL;
			mConfig   = NULL;
			mPower    = NULL;
			mStatus   = NULL;
			mCounter  = NULL;
			mDMA      = NULL;
			mI2C      = NULL;

			#if 2 <= PUBLIC_LEVEL
			mROM_     = NULL;
			#endif

			_ASSERT(mRegister == NULL); mRegister = new class Register(mPCI           ); _ASSERT(mRegister);
			_ASSERT(mConfig   == NULL); mConfig   = new class Config  (      mRegister); _ASSERT(mConfig  );
			_ASSERT(mPower    == NULL); mPower    = new class Power   (mPCI, mRegister); _ASSERT(mPower   );
			_ASSERT(mStatus   == NULL); mStatus   = new class Status  (      mRegister); _ASSERT(mStatus  );
			_ASSERT(mDMA      == NULL); mDMA      = new class DMA     (      mRegister); _ASSERT(mDMA     );
			_ASSERT(mI2C      == NULL); mI2C      = new class I2C     (      mRegister); _ASSERT(mI2C     );

			#if 2 <= PUBLIC_LEVEL
			_ASSERT(mROM_     == NULL); mROM_     = new class ROM_    (      mRegister); _ASSERT(mROM_    );
			#endif
		}

		~_()
		{
			delete_(mConfig);
			delete_(mPower );
			delete_(mStatus);
			delete_(mDMA   );
			delete_(mI2C   );

			#if 2 <= PUBLIC_LEVEL
			delete_(mROM_  );
			#endif

			delete_(mRegister);
		}

		Register *Register() const { return mRegister; }
		Config   *Config  () const { return mConfig; }
		Power    *Power   () const { return mPower; }
		Status   *Status  () const { return mStatus; }
		DMA      *DMA     () const { return mDMA; }
		I2C      *I2C     () const { return mI2C; }

		#if 2 <= PUBLIC_LEVEL
		ROM_     *ROM_    () const { return mROM_; }
		#endif

	protected:
		PCI::Device		*const mPCI;
		class Register	*mRegister;
		class Config	*mConfig;
		class Power		*mPower;
		class Status	*mStatus;
		class Counter	*mCounter;
		class DMA		*mDMA;
		class I2C		*mI2C;

		#if 2 <= PUBLIC_LEVEL
		class ROM_		*mROM_;
		#endif
};

}
}

#endif
