#ifndef _FPGA_Config_H
#define _FPGA_Config_H

#include "Prefix.h"
#include "FPGA_Register.h"
#include "EARTH_PT3.h"

namespace EARTH {
namespace FPGA {
	class Register;

	class Config {
	public:
		Config(FPGA::Register *register_) : mRegister(register_)
		{
			_ASSERT(mRegister);
		}

		~Config()
		{
		}

		struct Abort {
			bool	Target,
					Master;
		};

		void ResetAbort(const Abort *abort)
		{
			_ASSERT(abort);
			uint16	data =
				  (abort->Target ? 1 : 0) << 12
				| (abort->Master ? 1 : 0) << 13;

			mRegister->WriteConfig16(0x06, data);
		}

		void GetAbort(Abort *abort)
		{
			uint16	data;

			mRegister->ReadConfig16(0x06, &data);

			_ASSERT(abort);
			abort->Target = BIT_SHIFT_MASK(data, 12, 1) ? true : false;
			abort->Master = BIT_SHIFT_MASK(data, 13, 1) ? true : false;
		}

		struct Enable {
			bool	Memory,
					Master;
		};

#if 0
		status SetEnable(const Enable *enable)
		{
			_ASSERT(enable);
			uint16	data =
				  (enable->Memory ? 1 : 0) << 1
				| (enable->Master ? 1 : 0) << 2;

			status	status = mRegister->WriteConfig16(0x04, data);
			return status;
		}
#endif

		status GetEnable(Enable *enable)
		{
			uint16	data;
			status	status = mRegister->ReadConfig16(0x04, &data);
			if (status) return status;

			_ASSERT(enable);
			enable->Memory = BIT_SHIFT_MASK(data, 1, 1) ? true : false;
			enable->Master = BIT_SHIFT_MASK(data, 2, 1) ? true : false;
		}

		status SetAddress(uint32 address)
		{
			status	status = mRegister->WriteConfig32(0x10, address);
			return status;
		}

		status GetAddress(uint32 *address)
		{
			_ASSERT(address);
			status	status = mRegister->ReadConfig32(0x10, address);
			return status;
		}

	protected:
		FPGA::Register	*const mRegister;
	};
}
}

#endif
