#ifndef _FPGA_DMA_H
#define _FPGA_DMA_H

#include "Prefix.h"
#include "Gray.h"
#include "OS_Thread.h"
#include "FPGA_Register.h"
#include "EARTH_PT3.h"
#include <stdio.h>

namespace EARTH {
namespace FPGA {
	class Register;
	class Buffer;

	class DMA {
	public:
		DMA(FPGA::Register *register_) : mRegister(register_)
		{
			_ASSERT(mRegister);

			for (uint32 i=0; i<4; i++) {
				mAddress[i] = 0;
				mEnabled[i] = 0;
			}
		}

		~DMA()
		{
		}

		void GetTsError(uint32 index, uint32 *count)
		{
			uint32	address = 0x54 + 0x18*index;

			uint32	data;
			mRegister->Read32(address, &data);

			data = Gray::ToBinary(data, 32);

			_ASSERT(count);
			*count = data;
		}

		void ResetTsError(uint32 index)
		{
		}

		uint32 DataWhenClose() const
		{
			uint32 data = 0;//BIT_1(1) << BIT_CONTROL_END | BIT_1(1) << BIT_CONTROL_END_;

			return data;
		}

		void SetAddress(uint32 index, uint64 address)
		{
			mAddress[index] = address;
		}

		void SetEnabled(uint32 index, bool enabled)
		{
			uint32	address = BaseAddress(index);
			uint64	startAddress = mAddress[index];

			if (enabled) {
				mRegister->Write32(address+0x8, 1<<1);
				mRegister->Write64(address+0x0, startAddress);
				mRegister->Write32(address+0x8, 1<<0);
			} else {
#if 3 <= PUBLIC_LEVEL
				uint32	maxRefreshGray;
				mRegister->Read32(0x1c, &maxRefreshGray);
				uint32	maxRefresh = Gray::ToBinary(maxRefreshGray, 32);
				::wprintf(L"MaxRefresh: %d\n", maxRefresh);
#endif

				mRegister->Write32(address+0x8, 1<<1);

				while (true) {
					uint32	data;
					mRegister->Read32(address+0x10, &data);
					bool	busy = BIT_SHIFT_MASK(data, 0, 1) ? true : false;
					if (busy == false) break;
					OS::Thread::Sleep(1);
				}
			}

			mEnabled[index] = enabled;
		}

		void GetEnabled(uint32 index, bool *enabled)
		{
			_ASSERT(enabled);
			*enabled = mEnabled[index];
		}

		void SetTestMode(uint32 index, bool testMode = false, uint16 initial = 0, bool not = false, bool reset = false)
		{
			uint32	address = BaseAddress(index) + 0x0c;
			uint32	data = (reset ? 1 : 0) << 18 | (not ? 1 : 0) << 17 | (testMode ? 1 : 0) << 16 | initial << 0;

			_ASSERT(mRegister);
			mRegister->Write32(address, data);
		}

		void GetTransferInfo(uint32 index, PT::Device::TransferInfo *info)
		{
			uint32	address = BaseAddress(index) + 0x10;
			uint32	data;

			_ASSERT(mRegister);
			mRegister->Read32(address, &data);

			uint32	externalFIFO = BIT_SHIFT_MASK(data, 8, 10);

			_ASSERT(info);
			info->Busy                      = BIT_SHIFT_MASK(data,  0, 1) ? true : false;
			info->Status                    = BIT_SHIFT_MASK(data,  2, 4);
			info->InternalFIFO_A_Overflow   = BIT_SHIFT_MASK(data,  6, 1) ? true : false;
			info->InternalFIFO_A_Underflow  = BIT_SHIFT_MASK(data,  7, 1) ? true : false;
			info->ExternalFIFO_Overflow     = ((1<<10)-1) <= externalFIFO;
			info->ExternalFIFO_MaxUsedBytes = 512 * (1+externalFIFO);
			info->InternalFIFO_B_Overflow   = BIT_SHIFT_MASK(data, 18, 1) ? true : false;
			info->InternalFIFO_B_Underflow  = BIT_SHIFT_MASK(data, 19, 1) ? true : false;
		}

		void GetTsPinsLevel(uint32 index, PT::Device::TsPinsLevel *level)
		{
			uint32	address = 0x50 + index*0x18;

			uint32	data;
			mRegister->Read32(address, &data);

			_ASSERT(level);
			level->clock = BIT_SHIFT_MASK(data, 23, 1) ? true : false;
			level->data  = BIT_SHIFT_MASK(data, 20, 1) ? true : false;
			level->byte  = BIT_SHIFT_MASK(data, 21, 1) ? true : false;
			level->valid = BIT_SHIFT_MASK(data, 22, 1) ? true : false;
		}

		void GetTsSyncByte(uint32 index, uint8 *syncByte)
		{
			uint32	address = 0x50 + index*0x18;

			uint32	data;
			mRegister->Read32(address, &data);

			*syncByte = BIT_SHIFT_MASK(data, 24, 8);
		}

	protected:
		FPGA::Register	*const mRegister;
		uint64			mAddress[4];
		bool			mEnabled[4];

		uint32 BaseAddress(uint32 index)
		{
			return 0x40 + 0x18 * index;
		}
	};
}
}

#endif
