#ifndef _FPGA_Power_H
#define _FPGA_Power_H

#include "Prefix.h"
#include "OS_Thread.h"
#include "PCI_Device.h"
#include "FPGA_Register.h"
#include "EARTH_PT3.h"

namespace EARTH {
namespace FPGA {
	class Register;

	class Power {
	public:
		Power(PCI::Device *device, FPGA::Register *register_) : mDevice(device), mRegister(register_)
		{
			_ASSERT(mDevice);
			_ASSERT(mRegister);

			mLnbPower = mLnbPowerWhenClose = PT::Device::LNB_POWER_OFF;
		}

		~Power()
		{
		}

		status SetLnbPower(PT::Device::LnbPower lnbPower)
		{
			mLnbPower = lnbPower;
			
			uint32 data = Data(mLnbPower);
			mRegister->Write32(0x08, data);

			return PT::STATUS_OK;
		}

		status GetLnbPower(PT::Device::LnbPower *lnbPower) const
		{
			if (lnbPower == NULL) return PT::STATUS_INVALID_PARAM_ERROR;

			*lnbPower = mLnbPower;

			return PT::STATUS_OK;
		}

		status SetLnbPowerWhenClose(PT::Device::LnbPower lnbPower)
		{
			mLnbPowerWhenClose = lnbPower;

			uint32	data = Data(mLnbPowerWhenClose);

			_ASSERT(mDevice);
			mDevice->SetReset(data);

			return PT::STATUS_OK;
		}

		status GetLnbPowerWhenClose(PT::Device::LnbPower *lnbPower) const
		{
			if (lnbPower == NULL) return PT::STATUS_INVALID_PARAM_ERROR;

			*lnbPower = mLnbPowerWhenClose;

			return PT::STATUS_OK;
		}

	protected:
		PCI::Device				*const mDevice;
		FPGA::Register			*const mRegister;
		PT::Device::LnbPower	mLnbPower, mLnbPowerWhenClose;

		uint32 Data(PT::Device::LnbPower lnbPower) const
		{
			enum {
				BIT_LNB_UP,
				BIT_LNB_DOWN
			};
			
			uint32	data = 0;

			switch (lnbPower) {
				case PT::Device::LNB_POWER_OFF: data |= BIT_1(0) << BIT_LNB_UP | BIT_1(0) << BIT_LNB_DOWN; break;
				case PT::Device::LNB_POWER_15V: data |= BIT_1(1) << BIT_LNB_UP | BIT_1(1) << BIT_LNB_DOWN; break;
				case PT::Device::LNB_POWER_11V: data |= BIT_1(0) << BIT_LNB_UP | BIT_1(1) << BIT_LNB_DOWN; break;
				default                       : return PT::STATUS_GENERAL_ERROR;
			}

			data |= BIT_1(1) << 3 | BIT_1(1) << 2;

			return data;
		}
	};
}
}

#endif
