#ifndef _FPGA_Register_H
#define _FPGA_Register_H

#include "Prefix.h"
#include "PCI_Device.h"

namespace EARTH {
namespace FPGA {
	class _;

	class Register {
	public:
		Register(PCI::Device *device) : mDevice(device)
		{
			_ASSERT(mDevice);

			mWrite = false;
			mRead = false;
		}

		~Register()
		{
		}

		void EnableWrite()
		{
			mWrite = true;
		}

		void EnableRead()
		{
			mRead = true;
		}

		status WriteConfig8(uint32 address, uint8 data)
		{
			status status = mDevice->WriteConfig(address, &data, sizeof(data));
			return status;
		}

		status WriteConfig16(uint32 address, uint16 data)
		{
			status status = mDevice->WriteConfig(address, &data, sizeof(data));
			return status;
		}

		status WriteConfig32(uint32 address, uint32 data)
		{
			status status = mDevice->WriteConfig(address, &data, sizeof(data));
			return status;
		}

		status WriteConfig64(uint32 address, uint64 data)
		{
			status status = mDevice->WriteConfig(address, &data, sizeof(data));
			return status;
		}

		status ReadConfig8(uint32 address, uint8 *data)
		{
			_ASSERT(data);
			status status = mDevice->ReadConfig(address, data, sizeof(*data));
			return status;
		}

		status ReadConfig16(uint32 address, uint16 *data)
		{
			_ASSERT(data);
			status status = mDevice->ReadConfig(address, data, sizeof(*data));
			return status;
		}

		status ReadConfig32(uint32 address, uint32 *data)
		{
			_ASSERT(data);
			status status = mDevice->ReadConfig(address, data, sizeof(*data));
			return status;
		}

		status ReadConfig64(uint32 address, uint64 *data)
		{
			_ASSERT(data);
			status status = mDevice->ReadConfig(address, data, sizeof(*data));
			return status;
		}

		void GetBaseAddress(uint32 index, volatile void **ptr)
		{
			_ASSERT(mDevice);
			mDevice->GetBaseAddress(index, ptr);
		}

		void Write32(uint32 address, uint32 data)
		{
			_ASSERT((address % 4) == 0);
			_ASSERT(mWrite);

			WriteMemory(address, &data, sizeof(data));
		}

		void Write64(uint32 address, uint64 data)
		{
			_ASSERT(mWrite);

			WriteMemory(address, &data, sizeof(data));
		}

		void Read8(uint32 address, uint8 *data)
		{
			_ASSERT(mRead);

			_ASSERT(data);
			ReadMemory(address, data, sizeof(*data));
		}

		void Read32(uint32 address, uint32 *data)
		{
			_ASSERT((address % 4) == 0);
			_ASSERT(mRead);

			_ASSERT(data);
			ReadMemory(address, data, sizeof(*data));
		}

		void Read64(uint32 address, uint64 *data)
		{
			_ASSERT(mRead);

			_ASSERT(data);
			ReadMemory(address, data, sizeof(*data));
		}

	protected:
		PCI::Device		*const mDevice;
		bool			mWrite, mRead;

		void WriteMemory(uint32 address, const void *data, uint32 size)
		{
			volatile void	*ptr;
			GetBaseAddress(0, &ptr);

			volatile uint8	*ptr8 = static_cast<volatile uint8 *>(ptr);
			_ASSERT(ptr8);

			switch (size) {
				case 1 : *reinterpret_cast<volatile uint8  *>(ptr8 + address) = *static_cast<const uint8  *>(data); break;
				case 2 : *reinterpret_cast<volatile uint16 *>(ptr8 + address) = *static_cast<const uint16 *>(data); break;
				case 4 : *reinterpret_cast<volatile uint32 *>(ptr8 + address) = *static_cast<const uint32 *>(data); break;
				case 8 : *reinterpret_cast<volatile uint64 *>(ptr8 + address) = *static_cast<const uint64 *>(data); break;
				default: _ASSERT(false);                                                                            break;
			}
		}

		void ReadMemory(uint32 address, void *data, uint32 size)
		{
			volatile void	*ptr;
			GetBaseAddress(0, &ptr);

			volatile uint8	*ptr8 = static_cast<volatile uint8 *>(ptr);
			_ASSERT(ptr8);

			switch (size) {
				case 1 : *static_cast<uint8  *>(data) = *reinterpret_cast<volatile uint8  *>(ptr8 + address); break;
				case 2 : *static_cast<uint16 *>(data) = *reinterpret_cast<volatile uint16 *>(ptr8 + address); break;
				case 4 : *static_cast<uint32 *>(data) = *reinterpret_cast<volatile uint32 *>(ptr8 + address); break;
				case 8 : *static_cast<uint64 *>(data) = *reinterpret_cast<volatile uint64 *>(ptr8 + address); break;
				default: _ASSERT(false);                                                                      break;
			}
		}
	};
}
}

#endif
