#ifndef _PT_Bus_H
#define _PT_Bus_H

#include "Prefix.h"
#include "EARTH_PT3.h"
#include "PT_Device.h"
#include "PT_Device_.h"
#include <Windows.h>
#include "PCI_32_64.h"
#include "OS_Process.h"
#include "OS_Semaphore.h"
#include <vector>
#include <locale.h>

namespace EARTH {
namespace PCI { class _; }
namespace PT {

	class Device_;

	class _Bus : public Bus {
	public:
		_Bus()
		{
			::setlocale(LC_ALL, "japanese");

			mPCI = NULL;
			mSemaphore = NULL;
		}

		~_Bus()
		{
			delete_(mPCI);
			delete_(mSemaphore);
		}

		status Init()
		{
			_ASSERT(mSemaphore == NULL);
			mSemaphore = new OS::Semaphore(1);
			_ASSERT(mSemaphore);

			HMODULE library = ::LoadLibrary(TEXT("wdapi1100.dll"));
			if (library == NULL) return STATUS_WDAPI_LOAD_ERROR;
			::FreeLibrary(library);
			library = NULL;

			bool isWow64 = OS::Process::IsWow64();
			uint32 kernelBit = isWow64 ? 64 : 32;

			_ASSERT(mPCI == NULL);
			mPCI = static_cast<PCI::Top *>(new PCI::_Top(kernelBit));
			_ASSERT(mPCI);
			
			status	status;

			status = mPCI->Init();
			if (status) return status;

			return status;
		}

		virtual status Delete()
		{
			if (mDevice.size() != 0) return STATUS_ALL_DEVICES_MUST_BE_DELETED_ERROR;
			delete this;
			return STATUS_OK;
		}

		virtual status GetVersion(uint32 *version) const
		{
			if (version == NULL) return STATUS_INVALID_PARAM_ERROR;

			*version = 0x200;

			return STATUS_OK;
		}

		virtual status Scan(DeviceInfo *deviceInfoPtr, uint32 *deviceInfoCount)
		{
			if ((deviceInfoPtr == NULL) || (deviceInfoCount == NULL)) return STATUS_INVALID_PARAM_ERROR;

			if (mPCI == NULL) return STATUS_INTERNAL_ERROR;
			status status = mPCI->Bus()->ScanDevice(deviceInfoPtr, deviceInfoCount);

			return status;
		}

		virtual status NewDevice(const DeviceInfo *deviceInfoPtr, Device **device, Device_ **
			#if 2 <= PUBLIC_LEVEL
			device_
			#endif
			)
		{
			if ((deviceInfoPtr == NULL) || (device == NULL)) return STATUS_INVALID_PARAM_ERROR;

			if (mPCI == NULL) return STATUS_INTERNAL_ERROR;

			_Device		*_device = new _Device(DeleteDevice_, this, mPCI, deviceInfoPtr);
			*device = _device;

			mSemaphore->Acquire();
			mDevice.push_back(_device);
			mSemaphore->Release();

			#if 2 <= PUBLIC_LEVEL
			if (device_) {
				*device_ = _device->Device_();
			}
			#endif

			return STATUS_OK;
		}

		void DeleteDevice(_Device *device)
		{
			mSemaphore->Acquire();

			for (std::vector<_Device *>::iterator i=mDevice.begin(); i!=mDevice.end(); i++) {
				if (*i == device) {
					mDevice.erase(i);
					break;
				}
			}
			mSemaphore->Release();
		}

		static void DeleteDevice_(_Device *device, void *argument)
		{
			_Bus	*that = static_cast<_Bus *>(argument);
			_ASSERT(that);
			that->DeleteDevice(device);
		}

	protected:
		PCI::Top				*mPCI;
		OS::Semaphore			*mSemaphore;
		std::vector<_Device *>	mDevice;
	};

}
}

#endif
