#ifndef _PCI_Bus_BIT_H
#define _PCI_Bus_BIT_H

#include "PCI_Bus.h"
#include "EARTH_PT3.h"

#include <windows.h>
#include "windrvr.h"

namespace EARTH {
namespace PCI {
namespace KERNEL_BIT {

	class _Bus : public Bus {
	public:	
		_Bus() : mHandle(INVALID_HANDLE_VALUE)
		{
		}

		virtual ~_Bus()
		{
			if (mHandle != INVALID_HANDLE_VALUE) {
				WD_Close(mHandle);
				mHandle = INVALID_HANDLE_VALUE;
			}
		}

		virtual status Init()
		{
			const char	*NAME[] = {
				"windrvr6_EARTHSOFT_PT3"
			};

			status	status = PT::STATUS_GENERAL_ERROR;

			for (uint32 i=0; i<sizeof(NAME)/sizeof(*NAME); i++) {
				status = Init2(NAME[i]);
				if (status == PT::STATUS_OK) break;
			}

			return status;
		}

		virtual status ScanDevice(PT::Bus::DeviceInfo *deviceInfoPtr, uint32 *deviceInfoCount)
		{
			_ASSERT(deviceInfoPtr);
			_ASSERT(deviceInfoCount);
			uint32	inDeviceInfoCount = *deviceInfoCount;
			*deviceInfoCount = 0;

			WD_PCI_SCAN_CARDS	scan;
			BZERO(scan);

			const uint16	VENDOR_ID = 0x1172;

			scan.searchId.dwVendorId = VENDOR_ID;
			scan.searchId.dwDeviceId = 0x4c15;
			scan.dwOptions           = WD_PCI_SCAN_DEFAULT;

			DWORD	status = WD_PciScanCards(mHandle, &scan);
			if (status != WD_STATUS_SUCCESS) return PT::STATUS_WD_PciScanCards_ERROR;

			uint32	dstIndex = 0;
			for (uint32 i=0; i<scan.dwCards && dstIndex<inDeviceInfoCount; i++) {
				uint32	data[2];

				WD_PCI_SLOT		slot = scan.cardSlot[i];

				WD_PCI_CONFIG_DUMP	config;
				BZERO(config);

				config.pciSlot  = slot;
				config.dwBytes  = 4;
				config.fIsRead  = TRUE;

				bool	b = false;
				for (uint32 j=0; j<sizeof(data)/sizeof(*data); j++) {
					config.pBuffer  = &data[j];
					config.dwOffset = (j == 0) ? 0x2c : 0x08;
	
					DWORD	status = WD_PciConfigDump(mHandle, &config);
					b = (status == WD_STATUS_SUCCESS) && (config.dwResult == PCI_ACCESS_OK);
					if (b == false) break;
				}

				if (b) {
					const uint16	SUBSYSTEM_VENDOR_ID = static_cast<uint16>(~VENDOR_ID),
									SUBSYSTEM_ID        = 0x0368;
					const uint32	CLASS_CODE          = 0x048000;
					const uint8		REVISION_ID         = 0x01;

					const uint32	DATA[2] = {
						SUBSYSTEM_ID << 16 | SUBSYSTEM_VENDOR_ID << 0,
						CLASS_CODE   <<  8 | REVISION_ID         << 0
					};

					if (::memcmp(data, DATA, sizeof(DATA)) == 0) {
						deviceInfoPtr->Bus      = slot.dwBus;
						deviceInfoPtr->Slot     = slot.dwSlot;
						deviceInfoPtr->Function = slot.dwFunction;
						deviceInfoPtr->PTn      = 3;

						deviceInfoPtr++;
						dstIndex++;
						(*deviceInfoCount)++;
					}
				}
			}

			return PT::STATUS_OK;
		}

		virtual status WriteConfig(uint32 bus, uint32 slot, uint32 function, uint32 address, const void *data, uint32 size)
		{
			WD_PCI_SLOT		pciSlot;
			pciSlot.dwBus      = bus;
			pciSlot.dwSlot     = slot;
			pciSlot.dwFunction = function;

			WD_PCI_CONFIG_DUMP	config;
			BZERO(config);
			config.pciSlot  = pciSlot;
			config.pBuffer  = const_cast<void *>(data);
			config.dwOffset = address;
			config.dwBytes  = size;
			config.fIsRead  = FALSE;

			DWORD	status = WD_PciConfigDump(mHandle, &config);
			if ((status != WD_STATUS_SUCCESS) || (config.dwResult != PCI_ACCESS_OK)) return PT::STATUS_WD_PciConfigDump_ERROR;

			return PT::STATUS_OK;
		}

		virtual status ReadConfig(uint32 bus, uint32 slot, uint32 function, uint32 address, void *data, uint32 size)
		{
			WD_PCI_SLOT		pciSlot;
			pciSlot.dwBus      = bus;
			pciSlot.dwSlot     = slot;
			pciSlot.dwFunction = function;

			WD_PCI_CONFIG_DUMP	config;
			BZERO(config);
			config.pciSlot  = pciSlot;
			config.pBuffer  = data;
			config.dwOffset = address;
			config.dwBytes  = size;
			config.fIsRead  = TRUE;

			DWORD	status = WD_PciConfigDump(mHandle, &config);
			if ((status != WD_STATUS_SUCCESS) || (config.dwResult != PCI_ACCESS_OK)) return PT::STATUS_WD_PciConfigDump_ERROR;

			return PT::STATUS_OK;
		}

		virtual HANDLE Handle() const
		{
			return mHandle;
		}

	protected:
		HANDLE	mHandle;

		status Init2(const char *driverName)
		{
			const char	*name = WD_DriverName(driverName);
			if (name == NULL) return PT::STATUS_WD_DriverName_ERROR;

			_ASSERT(mHandle == INVALID_HANDLE_VALUE);
			mHandle = WD_Open();
			if (mHandle == INVALID_HANDLE_VALUE) return PT::STATUS_WD_Open_ERROR;
	
			status	status;

			WD_VERSION	version;
			BZERO(version);

			status = WD_Version(mHandle, &version);
			if ((status != WD_STATUS_SUCCESS) || (version.dwVer != WD_VER)) {
				WD_Close(mHandle);
				mHandle = INVALID_HANDLE_VALUE;
				return PT::STATUS_WD_Version_ERROR;
			}

			WD_LICENSE	license;
			BZERO(license);

			::strcpy_s(license.cLicense,
				#include "PCI_Bus_BIT_.h"
				);
			status = WD_License(mHandle, &license);
			if (status != WD_STATUS_SUCCESS) {
				WD_Close(mHandle);
				mHandle = INVALID_HANDLE_VALUE;
				return PT::STATUS_WD_License_ERROR;
			}

			return PT::STATUS_OK;
		}
	};

	status NewBus(Bus **bus_)
	{
		Bus		*bus = new _Bus;
	
		status	status = bus->Init();
		if (status) {
			delete_(bus);
		}

		_ASSERT(bus_);
		*bus_ = bus;

		return status;
	}
}
}
}

#endif
