#ifndef _PCI_Memory_BIT_H
#define _PCI_Memory_BIT_H

#include "PCI_Memory.h"
#include "PCI.h"
#include "EARTH_PT3.h"

#include <windows.h>
#include "windrvr.h"

#if defined(KERNEL_32BIT) || defined(KERNEL_64BIT)

namespace EARTH {
namespace PCI {
namespace KERNEL_BIT {

	using namespace EARTH;

	class _Memory : public Memory {
	public:	
		_Memory(HANDLE driver, DWORD card) : mDriver(driver), mCard(card)
		{
			_ASSERT(mDriver != INVALID_HANDLE_VALUE);
		}

		virtual ~_Memory()
		{
		}

		status Init()
		{
			return PT::STATUS_OK;
		}

		virtual void Delete()
		{
			delete this;
		}

		virtual status LockBuffer(void *ptr, uint32 size, Direction direction, void **handle)
		{
			uint32	page = (size + SIZE-1) / SIZE;

			uint32	page2;
			if (page <= WD_DMA_PAGES) {
				page2 = 0;
			} else {
				page2 = page - WD_DMA_PAGES;
			}

			uint32	dma_size = sizeof(WD_DMA) + sizeof(WD_DMA_PAGE)*page2;

			WD_DMA	*dma = static_cast<WD_DMA *>(::malloc(dma_size));
			if (dma == NULL) { _ASSERT(false); return PT::STATUS_GENERAL_ERROR; }
			::memset(dma, 0, dma_size);

			uint32	option = DMA_ALLOW_CACHE;
			if (direction & DIRECTION_WRITE) option |= DMA_FROM_DEVICE;
			if (direction & DIRECTION_READ ) option |= DMA_TO_DEVICE;
			if (0 < page2) {
				option |= DMA_LARGE_BUFFER;
			}

			#if defined(KERNEL_64BIT)
			option |= DMA_ALLOW_64BIT_ADDRESS;
			#endif

			_ASSERT(ptr);
			dma->pUserAddr = ptr;
			dma->dwBytes   = size;
			dma->dwOptions = option;
			dma->dwPages   = page;
			dma->hCard     = mCard;

			_ASSERT(mDriver != INVALID_HANDLE_VALUE);
			DWORD	status = WD_DMALock(mDriver, dma);
			if (status != WD_STATUS_SUCCESS) {
				::free(dma);
				dma = NULL;

				return PT::STATUS_WD_DMALock_ERROR;
			}

			_ASSERT(handle);
			*handle = dma;

			return PT::STATUS_OK;
		}

		virtual status UnlockBuffer(void *handle)
		{
			WD_DMA	*dma = static_cast<WD_DMA *>(handle);
			_ASSERT(dma);

			if (dma) {
				_ASSERT(mDriver != INVALID_HANDLE_VALUE);
				DWORD	status = WD_DMAUnlock(mDriver, dma);

				::free(dma);
				dma = NULL;

				if (status != WD_STATUS_SUCCESS) { _ASSERT(false); return PT::STATUS_WD_DMAUnlock_ERROR; }
			}

			return PT::STATUS_OK;
		}

		virtual status GetBufferInfo(void *handle, const PT::Device::BufferInfo **info, uint32 *count)
		{
			WD_DMA	*dma = static_cast<WD_DMA *>(handle);
			_ASSERT(dma);

			_ASSERT(info);
			*info = reinterpret_cast<const PT::Device::BufferInfo *>(dma->Page);

			_ASSERT(count);
			*count = dma->dwPages;

			return PT::STATUS_OK;
		}

		virtual status SyncCpu(void *handle)
		{
			WD_DMA	*dma = static_cast<WD_DMA *>(handle);
			_ASSERT(dma);

			DWORD	status = WD_DMASyncCpu(mDriver, dma);

			if (status != WD_STATUS_SUCCESS) { _ASSERT(false); return PT::STATUS_WD_DMASyncCpu_ERROR; }

			return PT::STATUS_OK;
		}

		virtual status SyncIo(void *handle)
		{
			WD_DMA	*dma = static_cast<WD_DMA *>(handle);
			_ASSERT(dma);

			DWORD	status = WD_DMASyncIo(mDriver, dma);

			if (status != WD_STATUS_SUCCESS) { _ASSERT(false); return PT::STATUS_WD_DMASyncIo_ERROR; }

			return PT::STATUS_OK;
		}

	protected:
		const HANDLE	mDriver;
		const DWORD		mCard;
	};

	status NewMemory(HANDLE driver, DWORD card, Memory **memory_)
	{
		Memory	*memory = new _Memory(driver, card);
		if (memory == NULL) return PT::STATUS_OUT_OF_MEMORY_ERROR;

		_ASSERT(memory_);
		*memory_ = memory;

		return PT::STATUS_OK;
	}
}
}
}

#endif

#endif
